Module: ProMotion::MapScreenModule
- Included in:
- MapScreen
- Defined in:
- lib/ProMotion/map/map_screen_module.rb
Defined Under Namespace
Modules: MapClassMethods
Constant Summary collapse
- PIN_COLORS =
{ red: UIColor.redColor, green: UIColor.greenColor, purple: UIColor.purpleColor }
Class Method Summary collapse
Instance Method Summary collapse
- #add_annotation(annotation) ⇒ Object
- #add_annotations(annotations) ⇒ Object
- #annotations ⇒ Object
- #center ⇒ Object
- #center=(params = {}) ⇒ Object
- #check_annotation_data ⇒ Object
- #clear_annotation(annotation) ⇒ Object
- #clear_annotations ⇒ Object
- #create_region(initialLocation, radius = 10) ⇒ Object (also: #region)
-
#deg_to_rad(angle) ⇒ Object
REGION SETTINGS #######.
- #deselect_annotations(animated = false) ⇒ Object
- #gesture_drop_pin(gesture_recognizer) ⇒ Object
- #hide_user_location ⇒ Object
- #look_up_address(args = {}, &callback) ⇒ Object
- #look_up_location(location, &callback) ⇒ Object
- #map ⇒ Object (also: #mapview)
-
#mapView(map_view, regionDidChangeAnimated: animated) ⇒ Object
Cocoa touch methods #################.
- #on_appear ⇒ Object
-
#point_from_location_bearing_and_distance(initialLocation, bearing, distance) ⇒ Object
Input coordinates and bearing in decimal degrees, distance in kilometers.
- #rad_to_deg(angle) ⇒ Object
- #screen_setup ⇒ Object
- #select_annotation(annotation, animated = true) ⇒ Object
- #select_annotation_at(annotation_index, animated = true) ⇒ Object
- #selected_annotations ⇒ Object
-
#set_map_style ⇒ Object
END REGION SETTINGS ###### MAP STYLE SETTINGS ######.
- #set_region(region, animated = true) ⇒ Object
- #set_show_user_location(show, mode = MGLUserTrackingModeFollow) ⇒ Object
- #set_start_position(params = {}) ⇒ Object
- #set_tap_to_add(params = {}) ⇒ Object
- #set_up_start_position ⇒ Object
- #set_up_tap_to_add ⇒ Object
- #show_user_location ⇒ Object
- #showing_user_location? ⇒ Boolean
-
#type ⇒ Object
Cocoa touch Ruby counterparts #################.
- #type=(type) ⇒ Object
- #update_annotation_data ⇒ Object
- #user_annotation ⇒ Object
- #user_location ⇒ Object
-
#zoom_to_fit_annotations(args = {}) ⇒ Object
TODO: Why is this so complex?.
- #zoom_to_user(radius = 0.05, animated = true) ⇒ Object
Class Method Details
.included(base) ⇒ Object
439 440 441 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 439 def self.included(base) base.extend(MapClassMethods) end |
Instance Method Details
#add_annotation(annotation) ⇒ Object
113 114 115 116 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 113 def add_annotation(annotation) @promotion_annotation_data << MapScreenAnnotation.new(annotation) self.view.addAnnotation @promotion_annotation_data.last end |
#add_annotations(annotations) ⇒ Object
118 119 120 121 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 118 def add_annotations(annotations) @promotion_annotation_data = Array(annotations).map{|a| MapScreenAnnotation.new(a)} self.view.addAnnotations @promotion_annotation_data end |
#annotations ⇒ Object
89 90 91 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 89 def annotations @promotion_annotation_data end |
#center ⇒ Object
39 40 41 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 39 def center self.view.centerCoordinate end |
#center=(params = {}) ⇒ Object
43 44 45 46 47 48 49 50 51 52 53 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 43 def center=(params={}) PM.logger.error "Missing #:latitude property in call to #center=." unless params[:latitude] PM.logger.error "Missing #:longitude property in call to #center=." unless params[:longitude] params[:animated] ||= true # Set the new region self.view.setCenterCoordinate( CLLocationCoordinate2D.new(params[:latitude], params[:longitude]), animated:params[:animated] ) end |
#check_annotation_data ⇒ Object
25 26 27 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 25 def check_annotation_data PM.logger.error "Missing #annotation_data method in MapScreen #{self.class.to_s}." unless self.respond_to?(:annotation_data) end |
#clear_annotation(annotation) ⇒ Object
123 124 125 126 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 123 def clear_annotation(annotation) self.view.removeAnnotation(annotation) @promotion_annotation_data.delete(annotation) end |
#clear_annotations ⇒ Object
128 129 130 131 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 128 def clear_annotations self.view.removeAnnotations(@promotion_annotation_data) @promotion_annotation_data = [] end |
#create_region(initialLocation, radius = 10) ⇒ Object Also known as: region
267 268 269 270 271 272 273 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 267 def create_region(initialLocation,radius=10) return nil unless initialLocation.is_a? CLLocationCoordinate2D radius = radius * 1.820 # Meters equivalent to 1 Nautical Mile southWest = self.point_from_location_bearing_and_distance(initialLocation,225, radius) northEast = self.point_from_location_bearing_and_distance(initialLocation,45, radius) {:southWest => southWest, :northEast => northEast} end |
#deg_to_rad(angle) ⇒ Object
REGION SETTINGS #######
236 237 238 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 236 def deg_to_rad(angle) angle*Math::PI/180 end |
#deselect_annotations(animated = false) ⇒ Object
105 106 107 108 109 110 111 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 105 def deselect_annotations(animated=false) unless selected_annotations.nil? selected_annotations.each do |annotation| self.view.deselectAnnotation(annotation, animated:animated) end end end |
#gesture_drop_pin(gesture_recognizer) ⇒ Object
168 169 170 171 172 173 174 175 176 177 178 179 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 168 def gesture_drop_pin(gesture_recognizer) if gesture_recognizer.state == UIGestureRecognizerStateBegan NSNotificationCenter.defaultCenter.postNotificationName("ProMotionMapWillAddPin", object:nil) touch_point = gesture_recognizer.locationInView(self.view) touch_map_coordinate = self.view.convertPoint(touch_point, toCoordinateFromView:self.view) add_annotation({ coordinate: touch_map_coordinate }.merge(@tap_to_add_annotation_params)) NSNotificationCenter.defaultCenter.postNotificationName("ProMotionMapAddedPin", object:@promotion_annotation_data.last) end end |
#hide_user_location ⇒ Object
63 64 65 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 63 def hide_user_location set_show_user_location false end |
#look_up_address(args = {}, &callback) ⇒ Object
286 287 288 289 290 291 292 293 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 286 def look_up_address(args={}, &callback) args[:address] = args if args.is_a? String # Assume if a string is passed that they want an address geocoder = CLGeocoder.new return geocoder.geocodeAddressDictionary(args[:address], completionHandler: callback) if args[:address].is_a?(Hash) return geocoder.geocodeAddressString(args[:address].to_s, completionHandler: callback) unless args[:region] return geocoder.geocodeAddressString(args[:address].to_s, inRegion:args[:region].to_s, completionHandler: callback) if args[:region] end |
#look_up_location(location, &callback) ⇒ Object
295 296 297 298 299 300 301 302 303 304 305 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 295 def look_up_location(location, &callback) location = CLLocation.alloc.initWithLatitude(location.latitude, longitude:location.longitude) if location.is_a?(CLLocationCoordinate2D) if location.kind_of?(CLLocation) geocoder = CLGeocoder.new geocoder.reverseGeocodeLocation(location, completionHandler: callback) else PM.logger.info("You're trying to reverse geocode something that isn't a CLLocation") callback.call nil, nil end end |
#map ⇒ Object Also known as: mapview
34 35 36 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 34 def map self.view end |
#mapView(map_view, regionDidChangeAnimated: animated) ⇒ Object
Cocoa touch methods #################
308 309 310 311 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 308 def mapView(mapView, annotationCanShowCallout: annotation) return nil if annotation.is_a? MGLUserLocation annotation.params[:show_callout] end |
#on_appear ⇒ Object
21 22 23 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 21 def on_appear update_annotation_data end |
#point_from_location_bearing_and_distance(initialLocation, bearing, distance) ⇒ Object
Input coordinates and bearing in decimal degrees, distance in kilometers
245 246 247 248 249 250 251 252 253 254 255 256 257 258 259 260 261 262 263 264 265 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 245 def point_from_location_bearing_and_distance(initialLocation, bearing, distance) distance = distance / 6371.01 # Convert to angular radians dividing by the Earth radius bearing = deg_to_rad(bearing) input_latitude = deg_to_rad(initialLocation.latitude) input_longitude = deg_to_rad(initialLocation.longitude) output_latitude = Math.asin( Math.sin(input_latitude) * Math.cos(distance) + Math.cos(input_latitude) * Math.sin(distance) * Math.cos(bearing) ) dlon = input_longitude + Math.atan2( Math.sin(bearing) * Math.sin(distance) * Math.cos(input_longitude), Math.cos(distance) - Math.sin(input_longitude) * Math.sin(output_latitude) ) output_longitude = (dlon + 3*Math::PI) % (2*Math::PI) - Math::PI CLLocationCoordinate2DMake(rad_to_deg(output_latitude), rad_to_deg(output_longitude)) end |
#rad_to_deg(angle) ⇒ Object
240 241 242 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 240 def rad_to_deg(angle) angle*180/Math::PI end |
#screen_setup ⇒ Object
10 11 12 13 14 15 16 17 18 19 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 10 def screen_setup self.view = nil self.view = MGLMapView.alloc.initWithFrame(self.view.bounds, styleURL: self.class.get_map_style) self.view.delegate = self check_annotation_data @promotion_annotation_data = [] set_up_start_position set_up_tap_to_add end |
#select_annotation(annotation, animated = true) ⇒ Object
93 94 95 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 93 def select_annotation(annotation, animated=true) self.view.selectAnnotation(annotation, animated:animated) end |
#select_annotation_at(annotation_index, animated = true) ⇒ Object
97 98 99 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 97 def select_annotation_at(annotation_index, animated=true) select_annotation(annotations[annotation_index], animated:animated) end |
#selected_annotations ⇒ Object
101 102 103 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 101 def selected_annotations self.view.selectedAnnotations end |
#set_map_style ⇒ Object
END REGION SETTINGS ###### MAP STYLE SETTINGS ######
278 279 280 281 282 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 278 def set_map_style if self.class.respond_to?(:get_map_style) self.view.styleURL = self.class.get_map_style end end |
#set_region(region, animated = true) ⇒ Object
228 229 230 231 232 233 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 228 def set_region(region, animated=true) self.view.setVisibleCoordinateBounds( [region[:southWest], region[:northEast]], animated: animated ) end |
#set_show_user_location(show, mode = MGLUserTrackingModeFollow) ⇒ Object
67 68 69 70 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 67 def set_show_user_location(show,mode=MGLUserTrackingModeFollow) self.view.showsUserLocation = show self.view.userTrackingMode = mode end |
#set_start_position(params = {}) ⇒ Object
133 134 135 136 137 138 139 140 141 142 143 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 133 def set_start_position(params={}) params = { latitude: 37.331789, longitude: -122.029620, radius: 10 }.merge(params) initialLocation = CLLocationCoordinate2D.new(params[:latitude], params[:longitude]) region = create_region(initialLocation,params[:radius]) set_region(region, animated:false) end |
#set_tap_to_add(params = {}) ⇒ Object
151 152 153 154 155 156 157 158 159 160 161 162 163 164 165 166 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 151 def set_tap_to_add(params={}) params = { length: 2.0, target: self, action: "gesture_drop_pin:", annotation: { title: "Dropped Pin", animates_drop: true } }.merge(params) @tap_to_add_annotation_params = params[:annotation] lpgr = UILongPressGestureRecognizer.alloc.initWithTarget(params[:target], action:params[:action]) lpgr.minimumPressDuration = params[:length] self.view.addGestureRecognizer(lpgr) end |
#set_up_start_position ⇒ Object
145 146 147 148 149 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 145 def set_up_start_position if self.class.respond_to?(:get_start_position) && self.class.get_start_position self.set_start_position self.class.get_start_position_params end end |
#set_up_tap_to_add ⇒ Object
181 182 183 184 185 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 181 def set_up_tap_to_add if self.class.respond_to?(:get_tap_to_add) && self.class.get_tap_to_add self.set_tap_to_add self.class.get_tap_to_add_params end end |
#show_user_location ⇒ Object
55 56 57 58 59 60 61 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 55 def show_user_location if location_manager.respondsToSelector('requestWhenInUseAuthorization') location_manager.requestWhenInUseAuthorization end set_show_user_location true end |
#showing_user_location? ⇒ Boolean
72 73 74 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 72 def showing_user_location? self.view.showsUserLocation end |
#type ⇒ Object
Cocoa touch Ruby counterparts #################
382 383 384 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 382 def type map.mapType end |
#type=(type) ⇒ Object
386 387 388 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 386 def type=(type) map.mapType = type end |
#update_annotation_data ⇒ Object
29 30 31 32 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 29 def update_annotation_data clear_annotations add_annotations annotation_data end |
#user_annotation ⇒ Object
80 81 82 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 80 def user_annotation self.view.userLocation.location.nil? ? nil : self.view.userLocation.location end |
#user_location ⇒ Object
76 77 78 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 76 def user_location user_annotation.nil? ? nil : user_annotation.coordinate end |
#zoom_to_fit_annotations(args = {}) ⇒ Object
TODO: Why is this so complex?
188 189 190 191 192 193 194 195 196 197 198 199 200 201 202 203 204 205 206 207 208 209 210 211 212 213 214 215 216 217 218 219 220 221 222 223 224 225 226 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 188 def zoom_to_fit_annotations(args={}) # Preserve backwards compatibility args = {animated: args} if args == true || args == false args = {animated: true, include_user: false}.merge(args) ann = args[:include_user] ? (annotations + [user_annotation]).compact : annotations #Don't attempt the rezoom of there are no pins return if ann.count == 0 #Set some crazy boundaries topLeft = CLLocationCoordinate2D.new(-90, 180) bottomRight = CLLocationCoordinate2D.new(90, -180) #Find the bounds of the pins ann.each do |a| topLeft.longitude = [topLeft.longitude, a.coordinate.longitude].min topLeft.latitude = [topLeft.latitude, a.coordinate.latitude].max bottomRight.longitude = [bottomRight.longitude, a.coordinate.longitude].max bottomRight.latitude = [bottomRight.latitude, a.coordinate.latitude].min end #Find the bounds of all the pins and set the map_view coord = CLLocationCoordinate2D.new( topLeft.latitude - (topLeft.latitude - bottomRight.latitude) * 0.5, topLeft.longitude + (bottomRight.longitude - topLeft.longitude) * 0.5 ) # Add some padding to the edges span = MKCoordinateSpanMake( ((topLeft.latitude - bottomRight.latitude) * 1.075).abs, ((bottomRight.longitude - topLeft.longitude) * 1.075).abs ) region = MKCoordinateRegionMake(coord, span) fits = self.view.regionThatFits(region) set_region(fits, animated: args[:animated]) end |
#zoom_to_user(radius = 0.05, animated = true) ⇒ Object
84 85 86 87 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 84 def zoom_to_user(radius = 0.05, animated=true) show_user_location unless showing_user_location? set_region(create_region(user_location,radius), animated) end |